==> pawm-1.9.7.tar.bz2: pawm-1.9.7/src/palabel.c <==
/*
    Filename: palabel.c
    Synopsis: PA Clickable Label with 3D effect.

                           Copyright (C) 2000,2001
            Ral Nez de Arenas Coronado <raul@pleyades.net>
	         David Gmez Espinosa <david@pleyades.net>

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of
    the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    See the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc.  59 Temple Place, Suite 330 Boston, MA 02111-1307 USA
*/

#include "palabel.h"


#define PALABEL_EVMASK (ButtonPressMask | ButtonReleaseMask |\
                        EnterWindowMask | LeaveWindowMask | ExposureMask)


/* Border width of PALabel. PLEASE DON'T CHANGE THIS */
#define BORDERW (2)



/* Data Structures *********************************************************/
static struct {
    GC gc;
    XFontStruct *font;
    unsigned int height;
    int baseline;
    struct {
        unsigned long background;
        unsigned long text;
        unsigned long textdim;
        unsigned long light;
        unsigned long shadow;
        unsigned long outline;
	unsigned long bgfocus;
	unsigned long textfocus;
	unsigned long lightfocus;
	unsigned long shadowfocus;
    } color;
} class;


struct sPALabel {          /* The Puto Amo Label */
    Window window;  
    char *text;
    int x;
    int y;
    Bool active;
    Bool pressd;
    Bool focused;
    unsigned int w;
    callback action;  /* What to do when PALabel pressed */
    void *param;      /* Param to the action */
};
#define PALabelBytes (sizeof(struct sPALabel))


static unsigned short int reference_count=0;


/* Function prototypes *****************************************************/
/* Handlers */
static void palabel_handler       (XEvent *, void *);
static void palabel_EnterNotify   (PALabel);
static void palabel_LeaveNotify   (PALabel);
static void palabel_Expose        (XExposeEvent *, PALabel);
static void palabel_ButtonPress   (XButtonPressedEvent *, PALabel);
static void palabel_ButtonRelease (XButtonPressedEvent *, PALabel);

/* Helpers */
static void draw_all (PALabel);
/****************************************************************************/




/****************************************************************************/
/*                                                                          */
/*                        Event handling routines                          */
/*                                                                          */
/****************************************************************************/
static void palabel_handler (XEvent *xevent, void *widget) {

    PALabel palabel=NULL;

    ASSERT(xevent);
    ASSERT(widget);
    
    palabel=(PALabel)widget;

    switch(xevent->type) {
	 
    case EnterNotify:                        
    	GOT(EnterNotify, PALabel);
        palabel_EnterNotify(palabel);
        break;
     
    case LeaveNotify:                       
     	GOT(LeaveNotify, PALabel);
        palabel_LeaveNotify(palabel);
        break;
     
    case ButtonPress:                        
    	GOT(ButtonPress, PALabel);
        palabel_ButtonPress(&(xevent->xbutton), palabel);
        break;

    case ButtonRelease:
     	GOT(ButtonRelease, PALabel);
        palabel_ButtonRelease(&(xevent->xbutton), palabel);
    	break;
     
    case Expose:                           
    	GOT(Expose, PALabel);
        palabel_Expose(&(xevent->xexpose), palabel);
        break;	

    default:
        SAY("Event %d on PALabel %ld",xevent->type, palabel->window); 

    }     
}



static void palabel_EnterNotify (PALabel palabel) {

    ASSERT(palabel);

    /* Make the label raise. Or something like that... */
    palabel->active=True;
    draw_all(palabel);

}



static void palabel_LeaveNotify (PALabel palabel) {

    ASSERT(palabel);    
    
    /* Remove palabel outline */
    palabel->active=False;
    draw_all(palabel);
}



static void palabel_Expose (XExposeEvent *event, PALabel palabel) {

    ASSERT(event);
    ASSERT(palabel);    

    if (event->count) return;
    draw_all(palabel);
}


static void palabel_ButtonPress (XButtonPressedEvent *event,
                                 PALabel palabel) {

    ASSERT(event);
    ASSERT(palabel);    

    if (event->button!=Button1) return;

    palabel->active=True;
    palabel->pressd=True;
    draw_all(palabel);
    
}



static void palabel_ButtonRelease (XButtonPressedEvent *event,
                                   PALabel palabel) {

    ASSERT(event);
    ASSERT(palabel);    

    if (event->button!=Button1) return;

    if (palabel->pressd==False) return;
    palabel->pressd=False;
    draw_all(palabel);

    if ((palabel->active==True) && (palabel->action))
        palabel->action(palabel->param);
}
/****************************************************************************/




/****************************************************************************/
/*                                                                          */
/*                      Private (static) functions                         */
/*                                                                          */
/****************************************************************************/
static void draw_all (PALabel palabel) {

    ASSERT(palabel);

    /* Draw outline */
    XSetFunction(display, class.gc, GXcopy);
    XSetSubwindowMode(display, class.gc, ClipByChildren);
 
    if (palabel->focused) 
        XSetWindowBackground(display, palabel->window, class.color.bgfocus);
    else
        XSetWindowBackground(display, palabel->window, class.color.background);
    XClearWindow(display, palabel->window);
    
    XSetForeground(display, class.gc, class.color.outline);
    XDrawRectangle(display, palabel->window, class.gc,
                   0, 0, palabel->w-1, class.height-1);

    /* Draw 3D border */ 
    if (palabel->pressd)
	if (palabel->focused)
            XSetForeground(display, class.gc, class.color.shadowfocus);
        else
            XSetForeground(display, class.gc, class.color.shadow);
    else
	if (palabel->focused)
            XSetForeground(display, class.gc, class.color.lightfocus);
        else
            XSetForeground(display, class.gc, class.color.light);
    XDrawLine(display, palabel->window, class.gc, 1, 1, (int) palabel->w-2, 1);
    XDrawLine(display, palabel->window, class.gc, 1, 1, 1, (int) class.height-2);
    
    if (palabel->pressd)
	if (palabel->focused)
            XSetForeground(display, class.gc, class.color.lightfocus);
        else
            XSetForeground(display, class.gc, class.color.light);
    else
	if (palabel->focused)
            XSetForeground(display, class.gc, class.color.shadowfocus);
        else
            XSetForeground(display, class.gc, class.color.shadow);
    XDrawLine(display, palabel->window, class.gc,
              (int) palabel->w-2, 2, (int) palabel->w-2, (int) class.height-2);
    XDrawLine(display, palabel->window, class.gc,
              2, (int) class.height-2, (int) palabel->w-2, (int) class.height-2);

    if (palabel->active && palabel->focused)
        XSetForeground(display, class.gc, class.color.textfocus);
    else if (palabel->active || palabel->focused)
	XSetForeground(display, class.gc, class.color.text);
    else
        XSetForeground(display, class.gc, class.color.textdim);

    if (palabel->focused)
        XSetBackground(display, class.gc, class.color.bgfocus);
    else
        XSetBackground(display, class.gc, class.color.background);
    XSetFunction(display, class.gc, GXcopy);

    if (palabel->text) {
        int x=0;
        int y=0;
        unsigned int w=0;
        size_t len=0;

        w=palabel->w-(2*BORDERW)-(2*(class.font->descent));
        len=x_trimtext(class.font, palabel->text, w);
	
        x=w-XTextWidth(class.font, palabel->text, (int) len);
        x/=2;
        x+=BORDERW + class.font->descent;
	
        y=class.baseline;
        if (palabel->pressd) x++, y++;
        XDrawString(display, palabel->window, class.gc,
                    x, y, palabel->text,(int) len);
    }
}
/****************************************************************************/




/****************************************************************************/
/*                                                                          */
/*                       Public (member) functions                         */
/*                                                                          */
/****************************************************************************/
void palabel_init (void) {

    if (reference_count++) return;

    class.gc=XCreateGC(display,ROOT,0,NULL);
    if (class.gc==None) BUG();
    class.color.background=x_color("label_base_color");
    class.color.text=x_color("label_text");
    class.color.textdim=x_color("label_text_inactive");
    class.color.light=x_color("label_highlight");
    class.color.shadow=x_color("label_shadow");
    class.color.outline=x_color("label_border");
    class.color.bgfocus=x_color("label_focus_background");
    class.color.textfocus=x_color("label_focus_text");
    class.color.lightfocus=x_color("label_focus_highlight");
    class.color.shadowfocus=x_color("label_focus_shadow");

    class.font=XLoadQueryFont(display, cfgpool_getstring(pawmpool,"label_font"));
    if (!class.font) BUG();
    
    class.height=class.font->ascent+(3*(class.font->descent))+(2*BORDERW);
    class.baseline=(class.height-(class.font->ascent+class.font->descent))/2;
    class.baseline+=class.font->ascent;
    //class.baseline=class.font->ascent+class.font->descent+BORDERW;

    XSetForeground(display, class.gc, BLACK);
    XSetFont(display, class.gc, class.font->fid);
    XSetGraphicsExposures(display, class.gc, False);
}




void palabel_done (void) {

    if (--reference_count) return;

    XFreeGC(display, class.gc);
    XFreeFont(display, class.font);
}




PALabel palabel_create (unsigned int w, callback action, void *param) {

    PALabel palabel;

    palabel=malloc(PALabelBytes);
    if (!palabel) die("Not enough memory");
 
    palabel->x=0;
    palabel->y=0;
    palabel->w=w;
    palabel->text=NULL;
    palabel->active=False;
    palabel->pressd=False;
    palabel->focused=False;
    palabel->action=action;
    palabel->param=param;

    /* Create the PALabel window */
    palabel->window=XCreateSimpleWindow(display, ROOT, 0, 0,
                                        palabel->w, class.height, 0,
                                        class.color.text,
                                        class.color.background);
    if (palabel->window==None) BUG();
    XStoreName(display, palabel->window, "palabel");
    XSelectInput(display, palabel->window, PALABEL_EVMASK);

    x_addsloot(palabel->window, palabel_handler, palabel);

    return palabel;
}




void palabel_destroy (PALabel palabel) {

    ASSERT(palabel);
	
    x_delsloot(palabel->window, palabel_handler, palabel);
    XDestroyWindow(display, palabel->window);

    /* Helps debugging */
    memset(palabel, '\0', PALabelBytes);
    free(palabel);
}




void palabel_map (PALabel palabel, unsigned int w) {

    ASSERT(palabel);

    palabel->w=w;
    XResizeWindow(display, palabel->window, palabel->w, class.height);
    XMapRaised(display, palabel->window);
}




void palabel_unmap (PALabel palabel) {

    ASSERT(palabel);
	
    XUnmapWindow(display, palabel->window);
}




void palabel_move (PALabel palabel, int x, int y) {
 
    ASSERT(palabel);
	
    palabel->x=x;
    palabel->y=y;

    XMoveWindow(display, palabel->window, x, y);
}




void palabel_reparent (PALabel palabel, Window parent, int x, int y) {

    ASSERT(palabel);
    ASSERT(parent!=None);

    palabel->x=x;
    palabel->y=y;

    XReparentWindow(display, palabel->window, parent, x, y);
}




void palabel_settext (PALabel palabel, const char *text, size_t len)
{
    ASSERT(palabel);	

    if (palabel->text) free(palabel->text);
    palabel->text=NULL;

    /* Assert some values */
    if (!text) return;
    palabel->text=malloc(len+1);
    if (!(palabel->text)) die("Not enough memory");

    strncpy(palabel->text, text, len);
    palabel->text[len]='\0';

    draw_all(palabel);  
}


void palabel_setfocus (PALabel palabel)
{
    ASSERT(palabel);

    palabel->focused=True;
    draw_all(palabel);
}


void palabel_unsetfocus (PALabel palabel)
{
    if (!palabel) return;

    palabel->focused=False;
    draw_all(palabel);
}


unsigned int palabel_height (void) 
{
    if (!reference_count) BUG();

    return (class.height);
}


unsigned int palabel_width (PALabel palabel)
{
    ASSERT(palabel);
	
    return (palabel->w);
}
/****************************************************************************/

==> pawm-2.2.2.tar.bz2: pawm-2.2.2/src/palabel.c <==
/*
    Filename: palabel.c
    Synopsis: PA Clickable Label with 3D effect.

                           Copyright (C) 2000-2006
            Ral Nez de Arenas Coronado <raul@pleyades.net>
	         David Gmez Espinosa <david@pleyades.net>

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of
    the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    See the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc.  59 Temple Place, Suite 330 Boston, MA 02111-1307 USA
*/

#include "palabel.h"
#include "pafont.h"


#define PALABEL_EVMASK (ButtonPressMask | ButtonReleaseMask |\
                        EnterWindowMask | LeaveWindowMask | ExposureMask)


/* Border width of PALabel. PLEASE DON'T CHANGE THIS */
#define BORDERW (2)
#define PALABEL_MARGIN 5



/* Data Structures *********************************************************/
static struct {
    GC gc;
    PAFont font;
    unsigned int height;
    int y;                /* Y coordinate to draw the label text */
    struct {
        unsigned long background;
        char *text;
        char *textdim;
        unsigned long light;
        unsigned long shadow;
        unsigned long outline;
	unsigned long bgfocus;
	char *textfocus;
	unsigned long lightfocus;
	unsigned long shadowfocus;
    } color;
} class;


struct sPALabel {          /* The Puto Amo Label */
    Window window;  
    char *text;
    int x;
    int y;
    Bool active;
    Bool pressd;
    Bool focused;
    unsigned int w;
    callback action;  /* What to do when PALabel pressed */
    void *param;      /* Param to the action */
};
#define PALabelBytes (sizeof(struct sPALabel))


static unsigned short int reference_count=0;


/* Function prototypes *****************************************************/
/* Handlers */
static void palabel_handler       (XEvent *, void *);
static void palabel_EnterNotify   (PALabel);
static void palabel_LeaveNotify   (PALabel);
static void palabel_Expose        (XExposeEvent *, PALabel);
static void palabel_ButtonPress   (XButtonPressedEvent *, PALabel);
static void palabel_ButtonRelease (XButtonPressedEvent *, PALabel);

/* Helpers */
static void draw_all (PALabel);
/****************************************************************************/




/****************************************************************************/
/*                                                                          */
/*                        Event handling routines                          */
/*                                                                          */
/****************************************************************************/
static void palabel_handler (XEvent *xevent, void *widget) {

    PALabel palabel=NULL;

    ASSERT(xevent);
    ASSERT(widget);
    
    palabel=(PALabel)widget;

    switch(xevent->type) {
	 
    case EnterNotify:                        
    	GOT(EnterNotify, PALabel);
        palabel_EnterNotify(palabel);
        break;
     
    case LeaveNotify:                       
     	GOT(LeaveNotify, PALabel);
        palabel_LeaveNotify(palabel);
        break;
     
    case ButtonPress:                        
    	GOT(ButtonPress, PALabel);
        palabel_ButtonPress(&(xevent->xbutton), palabel);
        break;

    case ButtonRelease:
     	GOT(ButtonRelease, PALabel);
        palabel_ButtonRelease(&(xevent->xbutton), palabel);
    	break;
     
    case Expose:                           
    	GOT(Expose, PALabel);
        palabel_Expose(&(xevent->xexpose), palabel);
        break;	

    default:
        SAY("Event %d on PALabel %ld",xevent->type, palabel->window); 

    }     
}



static void palabel_EnterNotify (PALabel palabel) {

    ASSERT(palabel);

    /* Make the label raise. Or something like that... */
    palabel->active=True;
    draw_all(palabel);

}



static void palabel_LeaveNotify (PALabel palabel) {

    ASSERT(palabel);    
    
    /* Remove palabel outline */
    palabel->active=False;
    draw_all(palabel);
}



static void palabel_Expose (XExposeEvent *event, PALabel palabel) {

    ASSERT(event);
    ASSERT(palabel);    

    if (event->count) return;
    draw_all(palabel);
}


static void palabel_ButtonPress (XButtonPressedEvent *event,
                                 PALabel palabel) {

    ASSERT(event);
    ASSERT(palabel);    

    if (event->button!=Button1) return;

    palabel->active=True;
    palabel->pressd=True;
    draw_all(palabel);
    
}



static void palabel_ButtonRelease (XButtonPressedEvent *event,
                                   PALabel palabel) {

    ASSERT(event);
    ASSERT(palabel);    

    if (event->button!=Button1) return;

    if (palabel->pressd==False) return;
    palabel->pressd=False;
    draw_all(palabel);

    if ((palabel->active==True) && (palabel->action))
        palabel->action(palabel->param);
}
/****************************************************************************/




/****************************************************************************/
/*                                                                          */
/*                      Private (static) functions                         */
/*                                                                          */
/****************************************************************************/
static void draw_all (PALabel palabel) {

    ASSERT(palabel);

    /* Draw outline */
    XSetFunction(display, class.gc, GXcopy);
    XSetSubwindowMode(display, class.gc, ClipByChildren);
 
    if (palabel->focused) 
        XSetWindowBackground(display, palabel->window, class.color.bgfocus);
    else
        XSetWindowBackground(display, palabel->window, class.color.background);
    XClearWindow(display, palabel->window);
    
    XSetForeground(display, class.gc, class.color.outline);
    XDrawRectangle(display, palabel->window, class.gc,
                   0, 0, palabel->w-1, class.height-1);

    /* Draw 3D border */ 
    if (palabel->pressd)
	if (palabel->focused)
            XSetForeground(display, class.gc, class.color.shadowfocus);
        else
            XSetForeground(display, class.gc, class.color.shadow);
    else
	if (palabel->focused)
            XSetForeground(display, class.gc, class.color.lightfocus);
        else
            XSetForeground(display, class.gc, class.color.light);
    XDrawLine(display, palabel->window, class.gc, 1, 1, (int) palabel->w-2, 1);
    XDrawLine(display, palabel->window, class.gc, 1, 1, 1, (int) class.height-2);
    
    if (palabel->pressd)
	if (palabel->focused)
            XSetForeground(display, class.gc, class.color.lightfocus);
        else
            XSetForeground(display, class.gc, class.color.light);
    else
	if (palabel->focused)
            XSetForeground(display, class.gc, class.color.shadowfocus);
        else
            XSetForeground(display, class.gc, class.color.shadow);
    XDrawLine(display, palabel->window, class.gc,
              (int) palabel->w-2, 2, (int) palabel->w-2, (int) class.height-2);
    XDrawLine(display, palabel->window, class.gc,
              2, (int) class.height-2, (int) palabel->w-2, (int) class.height-2);

    if (palabel->active && palabel->focused)
	pafont_setcolor(class.font, class.color.textfocus);
    else if (palabel->active || palabel->focused)
	pafont_setcolor(class.font, class.color.text);
    else
	pafont_setcolor(class.font, class.color.textdim);

    if (palabel->focused)
        XSetBackground(display, class.gc, class.color.bgfocus);
    else
        XSetBackground(display, class.gc, class.color.background);
    XSetFunction(display, class.gc, GXcopy);

    if (palabel->text) {
        int x=0;
        int y=0;
        unsigned int w=0;
        size_t len=0;

	/* Get usable width and len */
        w=palabel->w-(2*BORDERW)-(2*PALABEL_MARGIN);
        len=pafont_trimtext(class.font, palabel->text, w);
	
	/* Center text */
	x=(palabel->w-pafont_width(class.font, palabel->text, len))/2;
	
        y=class.y;
        if (palabel->pressd) x++, y++;
	
	pafont_draw(class.font, palabel->window, x, y, palabel->text, len);
    }
}
/****************************************************************************/




/****************************************************************************/
/*                                                                          */
/*                       Public (member) functions                         */
/*                                                                          */
/****************************************************************************/
void palabel_init (void) {

    if (reference_count++) return;

    class.gc=XCreateGC(display,ROOT,0,NULL);
    if (class.gc==None) BUG();
    
    class.color.background=x_color("label_base_color");
    class.color.text=x_colorstr("label_text");
    class.color.textdim=x_colorstr("label_text_inactive");
    class.color.light=x_color("label_highlight");
    class.color.shadow=x_color("label_shadow");
    class.color.outline=x_color("label_border");
    class.color.bgfocus=x_color("label_focus_background");
    class.color.textfocus=x_colorstr("label_focus_text");
    class.color.lightfocus=x_color("label_focus_highlight");
    class.color.shadowfocus=x_color("label_focus_shadow");

    class.font=pafont_create(display, "palabel");
    
    class.height=pafont_height(class.font)+((pafont_height(class.font)/2));
    class.y=(class.height-pafont_height(class.font))/2;

    XSetForeground(display, class.gc, BLACK);
    XSetGraphicsExposures(display, class.gc, False);
}




void palabel_done (void) {

    if (--reference_count) return;

    pafont_destroy(class.font);
    XFreeGC(display, class.gc);
}




PALabel palabel_create (unsigned int w, callback action, void *param) {

    PALabel palabel;

    palabel=malloc(PALabelBytes);
    if (!palabel) die("Not enough memory");
 
    palabel->x=0;
    palabel->y=0;
    palabel->w=w;
    palabel->text=NULL;
    palabel->active=False;
    palabel->pressd=False;
    palabel->focused=False;
    palabel->action=action;
    palabel->param=param;

    /* Create the PALabel window */
    palabel->window=XCreateSimpleWindow(display, ROOT, 0, 0,
                                        palabel->w, class.height, 0,
                                        0,
                                        class.color.background);
    if (palabel->window==None) BUG();
    XStoreName(display, palabel->window, "palabel");
    XSelectInput(display, palabel->window, PALABEL_EVMASK);

    x_addsloot(palabel->window, palabel_handler, palabel);

    return palabel;
}




void palabel_destroy (PALabel palabel) {

    ASSERT(palabel);
	
    x_delsloot(palabel->window, palabel_handler, palabel);
    XDestroyWindow(display, palabel->window);

    /* Helps debugging */
    memset(palabel, '\0', PALabelBytes);
    free(palabel);

	palabel=NULL;
}




void palabel_map (PALabel palabel, unsigned int w) {

    ASSERT(palabel);

    palabel->w=w;
    XResizeWindow(display, palabel->window, palabel->w, class.height);
    XMapRaised(display, palabel->window);
}




void palabel_unmap (PALabel palabel) {

    ASSERT(palabel);
	
    XUnmapWindow(display, palabel->window);
}




void palabel_move (PALabel palabel, int x, int y) {
 
    ASSERT(palabel);
	
    palabel->x=x;
    palabel->y=y;

    XMoveWindow(display, palabel->window, x, y);
}



void palabel_reparent (PALabel palabel, Window parent, int x, int y) {

    ASSERT(palabel);
    ASSERT(parent!=None);

    palabel->x=x;
    palabel->y=y;

    XReparentWindow(display, palabel->window, parent, x, y);
}




void palabel_settext (PALabel palabel, const char *text, size_t len)
{
    ASSERT(palabel);	

    if (palabel->text) free(palabel->text);
    palabel->text=NULL;

    /* Assert some values */
    if (!text) return;
    palabel->text=malloc(len+1);
    if (!(palabel->text)) die("Not enough memory");

    strncpy(palabel->text, text, len);
    palabel->text[len]='\0';

    draw_all(palabel);  
}


void palabel_setfocus (PALabel palabel)
{
    ASSERT(palabel);

    palabel->focused=True;
    draw_all(palabel);
}


void palabel_unsetfocus (PALabel palabel)
{
    ASSERT(palabel);

    palabel->focused=False;
    draw_all(palabel);
}


unsigned int palabel_height (void) 
{
    if (!reference_count) BUG();

    return (class.height);
}


unsigned int palabel_width (PALabel palabel)
{
    ASSERT(palabel);
	
    return (palabel->w);
}
/****************************************************************************/
